#
# Copyright(c) 2006-2019 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

# file : ts-common.sh
# This is intended to be included by all executable shell scripts in
# this package.
#
# NOTE : this "dots" dns.sh, so other scripts do not have to
#

# all scripts can assume these are set and usable
#
# INSTALL_DIR - determined by where the script was located
# DEBUG - indicates to echoMsg to also output DEBUG level messsages
# AUDIT_LOG - all echoMsg strings are logged here, regardless of $DEBUG
# *_LIST_PATH - qualify the block & allow lists paths under INSTALL_DIR
#
setGlobals() {

    local ORIG=$(pwd)
    if [ "$0" = "/bin/sh" ] || [ "$0" = "/opt/etc/rc.dtrace" ];then
        if [ "[${INSTALL_DIR}]" != "[]" ];then
            echo "setting INSTALL_DIR to environment variable: [$INSTALL_DIR].";
        else
            # try known installed dirs until one exists
            if [ -d '/var/home/threatstop' ];then
                INSTALL_DIR=/var/home/threatstop
            elif [ -d '/var/home/scripts/threatstop' ];then
                INSTALL_DIR=/var/home/scripts/threatstop
            elif [ -d '/opt/threatstop' ];then
                INSTALL_DIR=/opt/threatstop
            else
                INSTALL_DIR=$(dirname $1);
            fi
        fi
    else
        # echo "getting dirname of [$0]"
        cd $(dirname $0) # always where the script was run from
        INSTALL_DIR=$(pwd)
        cd ${ORIG}
    fi

    ADVANCED_MODE=0
    DEBUG=0
    FTP_URL=''

    for opt in $*; do
        # echoMsg DEBUG "checking $opt"
        case $opt in
            --advanced )       shift
                                    echo "::ADVANCED MODE::"
                                    ADVANCED_MODE=true
                                    ;;
            --debug | DEBUG )  shift
                                    echo "::IN DEBUG MODE::"
                                    DEBUG=true
                                    ;;
            --help | -h )           HELP=true ;; # check if $HELP and trigger usage()
            # -h | --help )           usage # usage should be done in device specific module
            #                         exit
            #                         ;;
            # * )                     usage
            #                         exit 1
        esac
        shift
    done

    AUDIT_LOG_DIR=${INSTALL_DIR}/audit
    AUDIT_LOG=${AUDIT_LOG_DIR}/$(basename $0).run.log
    mkdir -p ${AUDIT_LOG_DIR}
    if [ -e ${AUDIT_LOG} ]; then

        cp ${AUDIT_LOG} ${AUDIT_LOG}.bak
        echo -n "" >${AUDIT_LOG}
    fi

    # with the above, *NOW* echoMsg and myExit may be used

    # qualify path to lists
    if [ ! -d ${INSTALL_DIR}/lists ]; then

        mkdir -p ${INSTALL_DIR}/lists
        if [ $? -ne 0 ]; then

           myExit 1 ERROR "Unable to create lists directory"
        fi
    fi

    . ${INSTALL_DIR}/ts-dns.sh
}

# feedback to the user, and for audit logging
echoMsg() {

    local msg=""
    local dbg=""
    local old_ifs="${IFS}"
    local SILENT_UNLESS_DEBUG='START STOP WARN DOWN'
    # timestamp=`date +'%m-%d %H:%M:%S'` # timestamp should just be time
    # sine we print out the date to begin logging and finish logging.
    timestamp=`date +'[%H:%M:%S]'`
    IFS=
    case $1 in
        OK) msg="$timestamp `printf %-62s%s%s $2 '[OK]'`"
        ;;
        FAIL) msg="$timestamp `printf %-62s%s%s $2 '[FAIL]'`"
        ;;
        STOP) msg="$timestamp `printf %-62s%s%s $2 '[STOP]'`"
        ;;
        START) msg="$timestamp `printf %-62s%s%s $2 '[START]'`"
        ;;
        DOWN) msg="$timestamp `printf %-62s%s%s $2 '[DOWN]'`"
        ;;
        UP) msg="$timestamp `printf %-62s%s%s $2 '[UP]'`"
        ;;
        ERROR) msg="$timestamp `printf %-62s%s%s $2 '[ERROR]'`"
        ;;
        WARN) msg="$timestamp `printf %-62s%s%s $2 '[WARN]'`"
        ;;
        ALERT) msg="$timestamp `printf %-62s%s%s $2 '[ALERT]'`"
        ;;
        INFO) msg="$timestamp `printf %-62s%s%s $2`"
        ;;
        DISPLAY) echo -e "$timestamp $2"
                IFS="${old_ifs}"; # important before early exit
                return

        ;;
        NOLOG) if [ "$DEBUG" = true ]; then
                        echo -e "$timestamp `printf %-80s%s%s $2 '[NOLOG]'`";
                IFS="${old_ifs}"; # important before early exit
                return
                fi
        ;;

        DEBUG)
               # dbg="    [DEBUG]$timestamp : $2"
               # dbg="$timestamp DEBUG>> $2"
               dbg="$timestamp `printf %-80s%s%s $2 '[DEBUG]'`"
               if [ "$DEBUG" = true ]; then
                   msg=$dbg
               else
                   if [ "x${AUDIT_LOG}" != 'x' ]; then
                       echo -e $dbg >> ${AUDIT_LOG}
                   fi
               fi
        ;;
        *) msg="SCRIPT ERROR : Unrecognized parameter [$1]" ;;
    esac

    if [ "$msg" != "" ]; then
        if ! `echo $SILENT_UNLESS_DEBUG | grep -q -E "$1"` || [ "$DEBUG" = "true" ]; then
            echo -e $msg; # only want to print if not in SILENT_ON_DEBUG list
        fi


        if [ "x${AUDIT_LOG}" != 'x' ]; then
           echo -e $msg >> ${AUDIT_LOG}
        fi
    fi
    IFS="${old_ifs}"
}

# every script that uses these utilities should call this immediately after include this file
# returns
# 0 for success lock
# 1 unable to gain a lock; caller should not proceed
setLock() {

    if [ "$LOCKFILE" = "" ]; then

        echoMsg ERROR "LOCKFILE not set; the caller should set this first"
        exit 1
    fi

    echoMsg DEBUG "Locking current instance"
    if [ -e $LOCKFILE ]; then

        ps=$(cat $LOCKFILE)
        echoMsg ERROR "Failed to lock ... Another instance [$ps] of the same script is running"
        return 1
    fi

    # single quote the code snippet, so to evaluate $?
    # at trap time, not at time of trap definition;
    # this will exit with what was intended
    trap 'EXIT_STATUS=$?; /bin/rm -f $LOCKFILE; exit $EXIT_STATUS' 0 1 2 3 15

    echo $$ > $LOCKFILE
    return 0
}

# assumes LOCKFILE is set by the includer of these utilities
# includer should call this before exiting
unLock() {

    echoMsg DEBUG "Unlocking current instance"
    /bin/rm -f $LOCKFILE
}

# common exit point
# call with val level message, like os
# myExit 1 ERROR "some error message"
myExit() {

    local val=$1
    local level=$2
    local msg="$3"

    echoMsg $level "$msg"

    unLock
    exit $val
}

# determineSedProperties()
# bsd and gnu sed have some differences, the most notable
# being how to designate extended regular expression processing
#
# this will early-out, if defined in config file
#
# global set : SED_EXT_REGEXP
#
# returns :
# 0 for one found
# 1 for not found
determineSedProperties() {

    echoMsg START "Determining local SED regex flags"

    if [ "${SED_EXT_REGEXP}" != "" -a -x "${SED_EXT_REGEXP}" ]; then

        echoMsg DEBUG "SED_EXT_REGEXP (${SED_EXT_REGEXP}) is set in config file; will not probe for it"
        return 0
    fi

    echo foo | sed -E 'd;' > /dev/null 2>&1
    if [ $? -eq 0 ]; then

        echoMsg DEBUG "Found that -E is appropriate for this sed"
        SED_EXT_REGEXP='-E'
        return 0
    fi

    echo foo | sed -r 'd;' > /dev/null 2>&1
    if [ $? -eq 0 ]; then

        echoMsg DEBUG "Found that -r is appropriate for this sed"
        SED_EXT_REGEXP='-r'
        return 0
    fi

    echoMsg ERROR "Unable to determine required local sed options. Can not continue."
    return 1
}

# determineWebTool()
# find where the executable web tool exists, if any
#
# globals set :
#
# - WEB_TOOL
#
# Note, once this is successful, the user can use
# WEB_TOOL.  However, they're responsible for using
# correct flags for the task hand, and the tool that's
# set.
#
# returns :
# 0 for one found
# 1 for not found
determineWebTool() {

    local path
    local paths="/bin /sbin /usr/bin /usr/sbin"
    local tool
    local tools="curl fetch wget"

    echoMsg START "Determining Web Tool"

    if [ "${WEB_TOOL}" != "" -a -x "${WEB_TOOL}" ]; then

        echoMsg DEBUG "WEB_TOOL (${WEB_TOOL}) is set in config file; will not probe for it"
        return 0
    fi

    for tool in ${tools}; do

        for path in ${paths}; do

            if [ -x "${path}/${tool}" ]; then

                echoMsg DEBUG "setting WEB_TOOL to ${path}/${tool}"
                WEB_TOOL="${path}/${tool}"
                return 0
            fi
        done
    done

    echoMsg ERROR "Unable to find one of '${tools}'; should not continue"
    return 1
}

#
# returns
# 1. takes the passed variable name and value
# 2. validates the key/value pair
# 3. creates it if not there, or updates it if exists
# args :
#   - key - name of variable / config item
#   - value - value of that config item
#
# returns
# 0 for success
# 1 for failure (and may output ERROR information)
writeConfigItem() {

    local name=$1
    local value=$2
    local cfg
    local rval
    if [ "x${INSTALL_DIR}" = "x" ];then
        echoMsg WARN "No INSTALL_DIR defined to writeConfigItem";
    fi
    cfg=${INSTALL_DIR}/threatstop.conf

    # Validation
    # Note: the $3 value '0' defaults the return value to 0 if the key does not exist
    # in case we want to write something brand new to the config we haven't written validation for yet.
    # This will still fail if we pass in an invalid value for a key we have validation defined like PORT or TDID
    if validate_opt "$name" "$value" 0;then
        echoMsg NOLOG "[OK] [$name]=>'$value' validation";
    else
        echoMsg FAIL "[$name]=>'$value' validation";
        return 1
    fi

    action='Adding'
    test -e $cfg && grep -q "^${name}=" $cfg
    if [ $? -eq 0 ]; then

        action='Updating'
        # using PIPE char vs slash as this caused issues writing BASE64 encoded string (auto_key)
        sed -i.bak ${SED_EXT_REGEXP} -e "s|^${name}=\"?.+\"?$|${name}=\"${value}\"|;" $cfg
    else
        echo "${name}=\"$value\"" >> $cfg
    fi

    rval=$?
    if [ $rval -eq 0 ]; then
        echoMsg DEBUG "writeConfigItem $action ${name}=>\"${value}\" - $cfg Success"
    else
        echoMsg DEBUG "writeConfigItem $action ${name}=>\"${value}\" - $cfg Fail"
    fi

    return $rval
}

# args:
#     -name - config key / name
#     -default_value - if no value is found this is set in CONFIG_ITEM_VALUE
# returns:
#     0 - success
#     1 - fail
#     stores the value in ${CONFIG_ITEM_VALUE}
getConfigItem() {

    local name=$1
    local default_value=$2
    local cfg
    CONFIG_ITEM_VALUE=''; # unset for safety
    if [ "x${INSTALL_DIR}" = "x" ];then
        echoMsg WARN "No INSTALL_DIR defined to writeConfigItem";
    fi
    cfg=${INSTALL_DIR}/threatstop.conf
    if [ ! -f $cfg ];then
        echoMsg WARN "No config found $cfg";
        return 1
    fi

    parsed_value=`grep "^${name}=" ${cfg} | sed -n 's/^.*="\(.*\)"$/\1/p'`
    if NotEmpty $parsed_value; then
        CONFIG_ITEM_VALUE=$parsed_value
        echoMsg NOLOG "fetched [$name] value '$CONFIG_ITEM_VALUE'. Stored in CONFIG_ITEM_VALUE";
        return 0
    elif NotEmpty $default_value; then
        CONFIG_ITEM_VALUE=$default_value
        echoMsg NOLOG "defaulting [$name]=>'$CONFIG_ITEM_VALUE'"
        return 0
    else
        return 1
    fi


}

# loadConfig()
# populate the well-known variables, as follows
# 0. BAIL, if CONFIG_VERSION from config file doesn't match this
#    software version
# 1. load threatstop.conf (if it exists...if not, bail, since
#    the customer has yet to follow the cut-n-paste instructions
#    from the portal)
#    TSCM note : this will always exist
# 2. load threatstop.conf.template
#    This brings in default values, if needed (see next)
# 3. Examine critical variables, loading defaults from the template
#    if they do not yet exist.
# 4. One-off probes, like SRV, current system model, version, device IP, ...
#
# returns
# 0 for success
# 1 for failure
loadConfig() {

    local cfg=${INSTALL_DIR}/threatstop.conf
    local tpl=$cfg.template
    local target
    local old_ifs
    local srv_url
    local log_upload_url
    local valid_ip_url
    local port

    echoMsg START "Load Config"

    # load version
    TS_VERSION=$(cat ${INSTALL_DIR}/VERSION)
    echoMsg DEBUG "ThreatSTOP version ${VERSION}"

    if [ -e $cfg ]; then

         . $cfg # load all existing variables
    else
         echoMsg WARN "No prior configuration found; starting with default values"
    fi

    if [ "${CONFIG_VERSION}" = "" ]; then

        # initial install, so set current version in config file
        writeConfigItem CONFIG_VERSION ${TS_VERSION} || myExit 1 ERROR "Unable to set CONFIG_VERSION to ${TS_VERSION} in the configuration file"
    else

        # see if they're running new software on old config
        if [ "${CONFIG_VERSION}" != "${TS_VERSION}" ]; then

             echoMsg ERROR "You are attempting to run version ${TS_VERSION} software with a ${CONFIG_VERSION} configuration."
             echoMsg ERROR "You must first run \"ts-upgrade\"; this will perform required modifications."
             myExit 1 ERROR "Exiting..."
        fi
    fi

    if [ ! -e $tpl ]; then

         echoMsg ERROR "Cannot find $tpl; this should have been installed with this package"
         return 1
    fi

    # OPTIONAL TEMPLATE SUPPORT
    # For variables in the template file...the user of this script
    # *must* supply a threatstop.conf.template file, and it
    # *may* contain variables for the following code to process.
    # The processing will check for variable content, loading from
    # template value if there is no content; ie, an empty string.
    # This will know to process a variable by referencing the
    # TEMPLATE_VARS variable, provided in the template file.
    #
    # Mandatory interface to use this processing :
    # The template file must provide variable T, which is a prefix
    # string to each variable name.  The variables in the template
    # file must contain that prefix.  For example, if it is desired
    # to have FOO and BAR processed by the following loop, this
    # is an example of a compatible template file :
    #
    # TEMPLATE_VARS="FOO BAR"
    # T=myprefix_
    # myprefix_FOO="foo default value"
    # myprefix_BAR="bar default value"
    #
    if [ -e $tpl ]; then

        . $tpl # load all template variables, as well as the template prefix $T

        if [ "${TEMPLATE_VARS}" != "" ]; then

            local var
            local ivar
            local tvar
            for var in ${TEMPLATE_VARS}; do

                eval ivar=\$${var} # indirect value
                if [ "${ivar}" = "" ]; then

                    eval tvar=\$${T}${var} # get template value
                    eval "${var}=${tvar}"
                fi
            done
            BLOCK_LIST_PATH=${INSTALL_DIR}/${BLOCK_LIST_PATH}
            ALLOW_LIST_PATH=${INSTALL_DIR}/${ALLOW_LIST_PATH}
            # all template variables should be available now
        fi
    fi

    # see if we have dig, host, or other
    determineSedProperties
    if [ $? -ne 0 ]; then

        echoMsg ERROR "Will not continue without sed properties"
        exit 1
    fi

    # see if we have dig, host, or other
    determineLookupTool
    if [ $? -ne 0 ]; then

        echoMsg ERROR "Have no way to perform DNS lookups; exiting"
        exit 1
    fi

    # see if we have a web utility
    determineWebTool
    if [ $? -ne 0 ]; then

        echoMsg ERROR "Have no way to perform Web lookups; exiting"
        exit 1
    fi




    return 0
}



# DNS_override_config_and_test_connectivity
# it fetches SRV dns entries to override config URLs for validip, logupload etc..
# it also tests those new URLs, if they fail (juniper can't curl https urls) it tries
# an http version (via regex transform), if that fails it prompts user if we should proceed

DNS_override_config_and_test_connectivity() {
    # see if we need to update (both of the following use this SRV lookup)
    # 1. log upload URL
    # 2. valid IP check URL
    # see if we need to update the DNS server list
    echoMsg START "DNS config override"
    lookupSRV tsdns tcp threatstop.com
    if [ $? -eq 0 ]; then

        echoMsg DEBUG "DNS server SRV : ${SRV_OUT}"

        target=$(echo ${SRV_OUT} | awk '{print $4;}' | sed 's/\.$//;')
        if [ "${target}" != "${DNS_SERVER_LIST}" ]; then

            echoMsg DEBUG "Using ${target} for policy lookups"
            DNS_SERVER_LIST=${target}
            updateConfig DNS_SERVER_LIST ${DNS_SERVER_LIST}
        fi
    fi


    # see if we need to update the FTP upload URL
    lookupSRV logs sftp threatstop.com
    if [ $? -eq 0 ]; then

        echoMsg DEBUG "FTP URL SRV : ${SRV_OUT}"

        target=$(echo ${SRV_OUT} | awk '{print $4;}' | sed 's/\.$//;')
        srv_url="ftp://ftp@${target}/logs"
        if [ "${srv_url}" != "${FTP_URL}" ]; then

            echoMsg DEBUG "Will ftp logs to ${srv_url}"
            FTP_URL=${srv_url}
            updateConfig FTP_URL ${FTP_URL}
        fi
    fi


    lookupSRV logs https threatstop.com
    if [ $? -eq 0 ]; then

        echoMsg DEBUG "LOG UPLOAD URL SRV : ${SRV_OUT}"
        echoMsg DEBUG "VALID IP URL SRV : ${SRV_OUT}"

        target=$(echo ${SRV_OUT} | awk '{print $4;}' | sed 's/\.$//;')
        port=$(echo ${SRV_OUT} | awk '{print $3;}')
        log_upload_url="https://${target}:${port}/logupload.pl"
        if [ "${log_upload_url}" != "${LOG_UPLOAD_URL}" ]; then

            echoMsg DEBUG "Will upload logs to ${log_upload_url}"
            LOG_UPLOAD_URL=${log_upload_url}
            updateConfig LOG_UPLOAD_URL ${LOG_UPLOAD_URL}
        fi
        valid_ip_url="https://${target}:${port}/validip.pl"
        if [ "${valid_ip_url}" != "${VALID_IP_URL}" ]; then

            echoMsg DEBUG "Will check IPs at ${valid_ip_url}"
            VALID_IP_URL=${valid_ip_url}
            updateConfig VALID_IP_URL ${VALID_IP_URL}
        fi
        # echoMsg DEBUG "checkValidIP out: $exit_status Probed IP: $DEVICE_IP_PROBED"
        if checkValidIP; then
            echoMsg OK "Validating public IP access to ThreatSTOP"
        else
            echoMsg FAIL "Validating public IP access to ThreatSTOP"
            promptYesNo "Do you want to continue even though we can't connect to ThreatSTOP (yes/no)? (hit return for no) : " no
            if [ "${PROMPT_OUT}" = "yes" ]; then
                echoMsg DEBUG "user opted to proceed anyways"
            else
                echoMsg DEBUG "user opted to bail"
                return 1
            fi
        fi
    else
        echoMsg FAIL "Validating access to ThreatSTOP"
        promptYesNo "Do you want to continue even though we can't connect to ThreatSTOP (yes/no)? (hit return for no) : " no
        if [ "${PROMPT_OUT}" = "yes" ]; then
            echoMsg DEBUG "user opted to proceed anyways"
        else
            echoMsg DEBUG "user opted to bail"
            return 1
        fi
    fi

    return 0 # success

}

# updateConfig()
# set the named variable to the value provided; if it already
# exists, the value will be updated
#
# args
# - variable name
# - variable value
#
# returns
# 0 for success
# 1 for failure
updateConfig() {

    local cfg=${INSTALL_DIR}/threatstop.conf
    local name=$1
    local value="$2"

    echoMsg START "Updating Config"

    if [ -e "${cfg}" ]; then

        grep -q "^${name}=" $cfg
        if [ $? -eq 0 ]; then

            echoMsg DEBUG "updating $name to '$value'"
            sed -i.bak -e "s|^${name}=.*$|${name}=\"${value}\"|;" $cfg
        else

            echoMsg DEBUG "creating $name with '$value'"
            echo "${name}=\"${value}\"" >> $cfg
        fi
    else

        echoMsg DEBUG "creating $cfg"
        echoMsg DEBUG "creating $name with '$value'"
        echo "${name}=\"${value}\"" >> $cfg
    fi

    return $?
}

# simple sanity checks used
# take the item to check
# return
# 0 for success
# 1 for failure
confirmPolicy() {
    echo "$1" | grep -q -E '[^-.a-zA-Z0-9]' # looking for invalid chars
    if [ $? -eq 0 ]; then

        return 1
    else
        queryPolicyZone "$1"
        if [ $? -eq 0 ]; then

            return 0
        else

            return 1
        fi
    fi
}
confirmIP() {
    echo "$1" | grep -q -E '^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$' # positive match
    if [ $? -eq 0 ]; then

        return 0
    else

        return 1
    fi
}

confirmTDID() {
    echo "$1" | grep -q -E '^tdid_[0-9a-zA-Z]{8}$' # positive match
    if [ $? -eq 0 ]; then

        return 0
    else

        return 1
    fi
}

confirmAutoKey() {
    # should be base64 encoded. only valid chars are 10 digits, 26 lowercase characters, 26 uppercase characters as well as + and /
    echo "$1" | grep -q -E '^[a-zA-Z0-9\=\/\+]{80,90}$' # positive match between 80-90 chars
    if [ $? -eq 0 ]; then

        return 0
    else

        return 1
    fi

}

confirmInt() {
    echo "$1" | grep -q -E '^[0-9]+$' # positive match
    if [ $? -eq 0 ]; then

        return 0
    else

        return 1
    fi
}

confirmPort() {
    echo "$1" | grep -q -E '^[0-9]+$' # positive match
    if [ $? -eq 0 ]; then
        if [ $1 -le 65535 ] && [ $1 -gt 0 ];then
            return 0
        else
            return 1
        fi
    else
        return 1
    fi
}

Empty() {
    if [ "[$1]" = "[]" ] || [ "[$1]" = "[ ]" ];then
        return 0
    fi
    return 1
}

NotEmpty() {
    if ! Empty "$1";then
        return 0
    fi
    return 1
}

test_regex() {
    local input="$1"
    local test_regex="$2"
    echo "${input}" | grep -q -E "${test_regex}"
    if [ $? -ne 0 ]; then
        echo "error"
    else
        echo "ok"
    fi
}

# this is a function that should be over-riden in the device specific module where
# writeConfigItem or validate_opt is being called. See validate_opt for more details.
# validate_device_opt: takes $key $value pair and attempts to validate based on predefined tests
# if test does not exist returns 1 unless a default value is specified, then it returns that.
# - validate_device_opt is should be defined in the device specific lib for device specific checks
#
#
#   args:
#       -key - Variable Name / Configuration Key i.e. TDID, PORT, BLOCK_LIST
#       -value - value to be validated
#       -default_return - return value (defaults to 1 - fail)
#   returns:
#       -returns 0 - success
#       -returns 1 - failure
#
validate_device_opt() {
    local opt="$1";
    opt=`echo "$opt" | awk '{print tolower($0)}'`
    local value="$2";
    local default_rval="$3"
    local rval=1
    if [ "x$default_rval" != "x" ];then
        rval=$default_rval
    fi
    if [ "x$value" = "x" ];then
        echoMsg ERROR "Nothing to validate"
        return $rval
    else
        case $opt in
            # below are examples you can uncomment and repurpose.
            # soa)
            #     if confirmInt $value;then
            #         return 0
            #     fi
            #     return 1
            #     ;;
            # port)
            #     if confirmPort $value;then
            #         return 0
            #     fi
            #     return 1
            #     ;;
            # ftp_mode)
            #     if [ `test_regex $value '^passive$|^active$'` = "ok" ] ;then
            #         return 0
            #     fi
            #     return 1
            #     ;;
            *) echoMsg NOLOG "validate_device_opt: validation undefined for [$opt]. Can't validate '$value'";
                return $rval
            ;;
        esac
    fi
}

# validate_opt: takes $key $value pair and attempts to validate based on predefined tests
# if test does not exist it sends the same to a function called validate_device_opt
# which returns 1 unless a default value is specified, then it returns that.
# - validate_device_opt is should be defined in the device specific lib for device specific checks
#
#
#   args:
#       -key - Variable Name / Configuration Key i.e. TDID, PORT, BLOCK_LIST
#       -value - value to be validated
#       -default_return - return value (defaults to 1 - fail)
#   returns:
#       -returns 0 - success
#       -returns 1 - failure
#
validate_opt() {
    local opt="$1";
    opt=`echo "$opt" | awk '{print tolower($0)}'`
    local value="$2";
    local default_rval="$3"
    local rval=1
    if [ "x$default_rval" != "x" ];then
        rval=$default_rval
    fi
    if [ "x$value" = "x" ];then
        echoMsg ERROR "Nothing to validate"
        return 1
    else
        case $opt in
            soa)
                if confirmInt $value;then
                    return 0
                fi
                return 1
                ;;
            port)
                if confirmPort $value;then
                    return 0
                fi
                return 1
                ;;
            ftp_mode)
                if [ `test_regex $value '^passive$|^active$'` = "ok" ] ;then
                    return 0
                fi
                return 1
                ;;
            auto_key)
                if confirmAutoKey $value;then
                    return 0
                fi
                return 1
                ;;
            ctp_version)
                if confirmInt $value;then
                    return 0
                fi
                return 1
                ;;
            device_id)
                # if [ `test_regex $value '^tdid_[0-9a-zA-Z]{8}$'` = "ok" ] && confirmTDID $value;then
                if confirmTDID $value;then
                    return 0
                fi
                return 1
                ;;
            auto_type)
                if [ `test_regex $value '^auto$|^manual$'` = "ok" ];then
                    return 0
                fi
                return 1
                ;;
            block_list)
                if confirmPolicy $value;then
                    return 0
                fi
                return 1
                ;;
            allow_list)
                if confirmPolicy $value;then
                    return 0
                fi
                return 1
                ;;
            maxpolicysize)
                if confirmInt $value;then
                    return 0
                fi
                return 1
                ;;
            * )
                echoMsg NOLOG "validate_opt: validation undefined for [$opt] checking validate_device_opt";
                validate_device_opt $opt "$value" $rval
                return $?
                ;;
        esac
    fi

}
# Make sure the IP this device is seen as by ThreatSTOP
# is allowed to query.  The customer's portal configuration
# for this device drives ACLs that allow this IP.
#
# globals used :
# - WEB_TOOL
# - VALID_IP_URL
# globals set :
# - DEVICE_IP_PROBED is set with the IP as seen by the check CGI
#
# returns :
# 0 success
# 1 failure
checkValidIP() {

    local rval=0
    local valid_ip_out
    local valid_ip_ip
    local timeout=20; # in testing an EX I found 10seconds was too low and kept cutting off.
    local tool
    local tool_args

    case $(basename ${WEB_TOOL}) in
        curl) tool_args="--insecure --silent --connect-timeout ${timeout} ${VALID_IP_URL}"
        ;;
        fetch) tool_args="-T ${timeout} -q -o - ${VALID_IP_URL}"
        ;;
        wget)  tool_args="-T ${timeout} -q -O - ${VALID_IP_URL}"
        ;;
        *)  echoMsg ERROR "Cannot find a web tool with which to probe ${VALID_IP_URL}"
            return 1
        ;;
    esac

    echoMsg DEBUG "checkValidIP: ${WEB_TOOL} ${tool_args}"
    valid_ip_out=$(${WEB_TOOL} ${tool_args})
    if [ $? -eq 0 ]; then

        valid_ip_ip=$(echo "${valid_ip_out}" | sed -n ${SED_EXT_REGEXP} 's/^Your IP address: ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)$/\1/p;')
        echoMsg DEBUG "Query came from '${valid_ip_ip}'"

        echo "${valid_ip_out}" | grep -q 'Address is in the list of authorized hosts'
        if [ $? -eq 0 ]; then

            echoMsg DEBUG "${valid_ip_ip} is okay to query ThreatSTOP"
        else

            echoMsg DEBUG "${valid_ip_out}"
            rval=1
        fi
        echoMsg DEBUG "(${VALID_IP_URL})"
        echoMsg OK "ThreatSTOP Public IP Validation"
        DEVICE_IP_PROBED=${valid_ip_ip}
        if NotEmpty "${DEVICE_IP_PROBED}";then
            echoMsg DEBUG "Saving Probed IP as LOG_UPLOAD_IP: $DEVICE_IP_PROBED"
            # We should not be relying on this, but here for fallback if TDID not set.
            LOG_UPLOAD_IP=$DEVICE_IP_PROBED
            updateConfig LOG_UPLOAD_IP "$LOG_UPLOAD_IP"
        fi
    else
        # RETRY if VALID_IP_URL is https address since JUNIPER line can't fetch https addresses
        if echo $VALID_IP_URL| grep -q 'https:';then
            non_https_valid_ip_url=$(echo ${VALID_IP_URL} | sed 's/https\:/http\:/;'| sed 's/\:443//;')
            tool_args=$(echo $tool_args| sed 's/https\:/http\:/;'| sed 's/\:443//;')
            echoMsg WARN "failed validating access via ${VALID_IP_URL}, trying $non_https_valid_ip_url next...";
            VALID_IP_URL=$non_https_valid_ip_url
            valid_ip_out=$(${WEB_TOOL} ${tool_args})
            if [ $? -ne 0 ];then
                rval=1
            fi
            valid_ip_ip=$(echo "${valid_ip_out}" | sed -n ${SED_EXT_REGEXP} 's/^Your IP address: ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)$/\1/p;')
            updateConfig VALID_IP_URL "$VALID_IP_URL"
            updateConfig LOG_UPLOAD_IP "$valid_ip_ip"
            DEVICE_IP_PROBED=${valid_ip_ip}
            echoMsg DEBUG "Probed IP: $DEVICE_IP_PROBED"
        else
            echoMsg DEBUG "(${VALID_IP_URL})"
            echoMsg WARN "ThreatSTOP Public IP Validation [FAIL]"
            rval=1
        fi
    fi


    return $rval
}

# these get run when this utility code is "dotted"
setGlobals $@
# this is now handled; let's cleanup the command line


# remove long --opts from positional params
# echoMsg DEBUG "before ARGS: $*";
for opt in $@; do
    if [ `echo "$opt"|grep -c '\-\-'` -gt 0 ]; then

        echoMsg NOLOG "Removing long-style opt ($opt) from command-line args"
    else
        # unfortunately isn't passed spaced tokens i.e. "file name.txt"
        # if [ `echo "$opt"|grep -c ' '` -gt 0 ]; then
        #     new_args="${new_args} \"$opt\""
        # fi
        # echoMsg DEBUG "passing $opt along";
        new_args="${new_args} $opt"
    fi
    shift
done
# echoMsg DEBUG "ARGS after shifts: $*";
set -- ${new_args}
# echoMsg DEBUG "ARGS after set: $*";

