#
# Copyright(c) 2006-2016 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

# file : ts-dns.sh
# A module to centralize all DNS lookups, including queries for
# policy PTR/APL records, SRV lookups, etc.
#

# performLookup() :
#
# "the mother of all lookup functions"
#
# All lookups should come through this wrapper.  This takes into
# account whether "dig" or "host" is being used, as well as to optionally
# perform recursive lookups (ie, NOT using the ThreatSTOP servers).
#
# args :
# - type (PTR, APL, SRV, ...)
# - fqdn (host or ip)
# - dig_options (mandatory options for "dig"...can be empty string)
# - dig_filter (mandatory sed filter for the "dig" output...can be empty string)
# - host_options (mandatory options for "host"...can be empty string)
# - host_filter (mandatory sed filter for the "host" output...can be empty string)
# - query_ts_servers ( true means yes, false means query general DNS )
#
# globals referenced :
# - LOOKUP (will contain full path to either "dig" or "host")
# - PROTO (udp or tcp)
# - PORT
# - DNS_SERVER_LIST
#
# upon success :
#     - loads LOOKUP_OUT with query results
#     - returns 0
# upon failure :
#     - loads LOOKUP_OUT with empty string
#     - returns 1
#
performLookup() {

    local type=$1
    local fqdn=$2
    local dig_options="$3"
    local dig_filter="$4"
    local host_options="$5"
    local host_filter="$6"
    local query_ts_servers=$7

    local myproto
    local old_ifs
    local server
    local use_dig=false
    local atserver=""
    local rval=1
    local dig_sed='p;'  # by default, print all (ie, no filtering)
    local host_sed='p;' # by default, print all (ie, no filtering)
    LOOKUP_OUT=""

    echo ${LOOKUP} | grep -q dig
    if [ $? -eq 0 ]; then

        use_dig=true
    fi

    if [ "${dig_filter}" != "" ]; then

        dig_sed="${dig_filter}"
    fi
    if [ "${host_filter}" != "" ]; then

        host_sed="${host_filter}"
    fi

    # go with first server where we have a successful lookup
    for server in ${DNS_SERVER_LIST}; do

        if [ "$use_dig" = "true" ]; then
            if ! confirmIP $server; then
                # echoMsg DEBUG "looking up IP address for $server"
                server_ip=$(${LOOKUP} +short $server | sed 1q);
                # echoMsg DEBUG "resolved ip: '$server_ip'"
                if confirmIP $server_ip
                then
                    echoMsg DEBUG "Using IP address $server_ip for $server"
                    server=$server_ip
                fi
            fi

            # done
            myproto="+${PROTO}"
            if [ "$query_ts_servers" = "true" ]; then
                atserver="@${server}"
            fi

            # dig
            echoMsg DEBUG "${LOOKUP} $dig_options +short $myproto -t $type -p ${PORT} $atserver $fqdn | sed -n ${SED_EXT_REGEXP} \"$dig_sed\""
            LOOKUP_OUT=$(${LOOKUP} $dig_options +short $myproto -t $type -p ${PORT} $atserver $fqdn)
            rval=$?
            LOOKUP_OUT=$(echo "${LOOKUP_OUT}" | sed -n ${SED_EXT_REGEXP} "$dig_sed")
        else

            myproto=""
            if [ ${PROTO} = "tcp" ]; then

                myproto="-T"
            fi

            if [ "$query_ts_servers" = "true" ]; then

                atserver=$server
            fi

            # host - does NOT offer port customization
            echoMsg DEBUG "${LOOKUP} $host_options $myproto -t $type $fqdn $atserver | sed -n ${SED_EXT_REGEXP} \"$host_sed\""
            LOOKUP_OUT=$(${LOOKUP} $host_options $myproto -t $type $fqdn $atserver)
            rval=$?
            LOOKUP_OUT=$(echo "${LOOKUP_OUT}" | sed -n ${SED_EXT_REGEXP} "$host_sed")
        fi

        if [ $rval -eq 0 ]; then

            break;
        fi
    done

    if [ $rval -eq 0 ]; then

        echoMsg DEBUG "Lookup successful"
    else

        echoMsg ERROR "Lookup failed"
        LOOKUP_OUT=""
    fi

    return $rval
}

# determineLookupTool - examine the query commands that exist on the
#              system and load the LOOKUP variable with the path
#              to the preferred command.  Even though some models
#              may have both the "dig" and "host" commands, this code
#              will prefer "dig", because it supports port customization.
#
# globals
#   sets LOOKUP
#
# NOTE : will bail early, if LOOKUP is already set; this check
# can be avoided by defining LOOKUP in the config file.
#
# returns
# 0 for success
# 1 for failure to find either
determineLookupTool() {

    local path
    local paths="/bin /sbin /usr/bin /usr/sbin"
    local lookup
    local lookups="dig host"

    echoMsg DEBUG "determineLookupTool():"

    if [ "${LOOKUP}" != "" -a -x "${LOOKUP}" ]; then

        echoMsg DEBUG "LOOKUP (${LOOKUP}) is set in config file; will not probe for it"
        return 0
    fi

    for lookup in ${lookups}; do

        for path in ${paths}; do

            if [ -x "${path}/${lookup}" ]; then

                echoMsg DEBUG "setting LOOKUP to ${path}/${lookup}"
                LOOKUP="${path}/${lookup}"
                return 0
            fi
        done
    done

    echoMsg ERROR "Unable to find one of '${lookups}'; should not continue"
    return 1
}

#
# lookupSRV - given the passed domain, service and protocol, return the
#              first SRV found.  NOTE : pass in the values *without* the
#              leading underscore.
#
# returns :
# upon success :
#     - loads SRV_OUT with space-separated : PRIO WEIGHT PORT TARGET
#     - returns 0
# upon failure :
#     - loads SRV_OUT with empty string
#     - returns 1
#
lookupSRV() {

    local service=$1  # sans leading underscore
    local protocol=$2 # sans leading underscore
    local domain=$3
    local srv
    local old_ifs
    local rval=1
    SRV_OUT=""

    echoMsg DEBUG "lookupSRV(): $service, $protocol, $domain"

    # when "host" is doing the querying, the result has some preceding stuff that needs to get
    # removed; the important 4 parts at the end are identical between "dig" and "host"
    # we also grab the first one we find
    performLookup srv _${service}._${protocol}.${domain} "+time=5" 'p;' "" 's/^[^ ]+ has SRV record //p;' "false"
    if [ $? -eq 0 ]; then

        srv=${LOOKUP_OUT}
        # we only want the first one
        old_ifs="${IFS}"
        IFS=
        srv=$(echo ${srv} | sed -n '1p;')
        if [ $(echo ${srv} | wc -w) -eq 4 ]; then

            SRV_OUT="$srv"
            rval=0
        fi
        IFS="${old_ifs}"
    else
        echoMsg ERROR "Could not lookup SRV records for ${domain}"
    fi

    return $rval
}

# queryPolicyZone()
# it is query-able for one or more PTR records
#
# args
# - policy zone
#
# returns
# 0 - query returned at least on PTR record
# 1 - no PTR, or error
queryPolicyZone() {

    local zone=$1

    performLookup ptr "${zone}" "" "" "" 's/^[^ ]+ domain name pointer //p;' "true"
    if [ $? -ne 0 -o "${LOOKUP_OUT}" = "" ]; then

        echoMsg ERROR "No records available for '${zone}'"
        return 1
    fi

    return 0
}
