#!/bin/sh
#
# Copyright(c) 2006-2019 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

# FILE : ts-junos.sh
# This is where all communications with the device takes place.

USAGE=$(cat <<EOU


    USAGE: root@juniper% /bin/sh tsadmin [ add | remove | update | show | version | revert ]
        'add' subcommand optional parameters: [-a allowlist] [-b blocklist] [-f firewall_mode]
             [-d Device ID] [-m max policy size] [-h help] [-x ftp_mode] [-p port]

        The 'add' subcommand initiates installation. It's arguments are optional.
        When provided they seed the interactive prompts with default values.
        When ALL non-defaulting arguments are supplied, the session will be non-interactive,
        and the system will be configured with the validated arguments.

        [allowlist, blocklist]
        - represent the fully qualified domain names of the
          portal-configured policies for allow and block (respectively)

        [firewall_mode]
        - "filter" for filter/prefix-lists, "zone" for zone/address-books
          NOTE : "zone" is not yet supported at this time.

        [ftp_mode]
        - This option allows you to set "active" or "passive" for log
        transfers. *defaults to 'active'

        [port]
        - This option allows you to set the DNS port for upstream lookups.
            Valid choices are (53 / 5353) *defaults to 53

        [device id]
        - a portal-provided identifier with which uploaded log data
        will be associated on the ThreatSTOP backend servers.

        [max policy size]
        - an integer value at which the total number of CIDR elements in
          a prefix list will be capped.

        [help]
        - present this usage statement
EOU
)


# all CLI calls should come through this helper
# args :
# - string of all commands to be issued to the CLI
#
# globals
# - NICE will influence whether the cli command is
#   run at lowest cpu scheduling priority
# - CLI_OUTPUT - gets loaded with the output (stdout
#   and stderr) from the command run here.
#   If this function returns success, this variable
#   can be parsed for desired output.  Upon failure,
#   this variable is noted here at the ERROR level.
#
# returns
# 0 - successful issue of CLI command
# 1 - problems encountered issuing CLI command
callCLI() {

    local cmd="$1"
    local rval=0

    # echoMsg DEBUG "callCLI():"

    CLI_OUTPUT=""

    # echoMsg DEBUG "sending the following to CLI"
    echoMsg DEBUG "callCLI: $cmd"
    CLI_OUTPUT=$(echo -e "$cmd" | /bin/csh -c "nice ${NICE} /usr/sbin/cli" 2>&1)

    if [ $? -eq 0 ]; then

        echoMsg DEBUG "callCLI [success]"
    else

        echoMsg ERROR "callCLI [fail]"
        rval=1
    fi
    echoMsg DEBUG "callCLI output: ${CLI_OUTPUT}"

    return $rval
}

# soleConfigurator()
# confirm that there are no other configure sessions in progress
#
# returns
# 0 this session is the only configure that is in progress
# 1 other configure(s) are in progress; we must error out
soleConfigurator() {

    local output=""
    local rval=0

    callCLI "configure\nexit"
    if [ $? -eq 0 ]; then

        output=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} '/^Users currently editing/p; /\(pid [0-9]+\) on since/p;')
        if [ "${output}" != "" ]; then

            echoMsg ERROR "${output}"
            return 1
        else

            echoMsg DEBUG "There are no other configuration sessions"
        fi

        output=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} 's/.*changed but not committed$/Configuration changes elsewhere are staged, but not committed/p;' | sort -u)
        if [ "${output}" != "" ]; then

            echoMsg ERROR "${output}"
            rval=1
        else

            echoMsg DEBUG "There are no other uncommitted changes"
        fi
    fi

    return $rval
}

# createGroup()
# This is called when it has been determined that it does
# not yet exist.  All aspects *except* IP addresses
# (prefix lists and address books) are created here, including
# 1. syslog
# 2. prefix lists
#
# returns :
# 0 for success
# 1 for failure
createGroup() {

    local header
    local firewall_ex
    local firewall_mx
    local firewall_srx
    local zone
    local footer
    local all_cmds
    local rval=0

    echoMsg DEBUG "createGroup():"

    if Empty ${DEVICE_ID}; then
        if Empty "${LOG_UPLOAD_IP}"; then
            echoMsg ERROR "No log identifier found. Can not proceed."
            return 1
        else
            echoMsg DEBUG "Using LOG_UPLOAD_IP $LOG_UPLOAD_IP as log identifier";
            LOG_FILENAME=${LOG_UPLOAD_IP}
        fi
    else
        echoMsg DEBUG "Using DEVICE_ID $DEVICE_ID as log identifier";
        LOG_FILENAME=${DEVICE_ID}
    fi

    header=$(cat <<EOH
configure
edit groups ${TS_GROUP_NAME}
delete system syslog
set system syslog file ${LOG_FILENAME}.log archive size ${SYSLOG_SIZE}k transfer-interval ${SYSLOG_INTERVAL} archive-sites ${FTP_URL}
set system syslog file ${LOG_FILENAME}.log structured-data brief
EOH
)

    firewall_ex=$(cat <<EOF
set system syslog file ${LOG_FILENAME}.log firewall any
set policy-options prefix-list ${ALLOW_PREFIX_LIST}
set policy-options prefix-list ${BLOCK_PREFIX_LIST}
set firewall family inet filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST}-src from source-prefix-list ${ALLOW_PREFIX_LIST}
set firewall family inet filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST}-src then syslog
set firewall family inet filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST}-src then accept
set firewall family inet filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST}-dst from destination-prefix-list ${ALLOW_PREFIX_LIST}
set firewall family inet filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST}-dst then syslog
set firewall family inet filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST}-dst then accept
set firewall family inet filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST}-src from source-prefix-list ${BLOCK_PREFIX_LIST}
set firewall family inet filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST}-src then syslog
set firewall family inet filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST}-src then discard
set firewall family inet filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST}-dst from destination-prefix-list ${BLOCK_PREFIX_LIST}
set firewall family inet filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST}-dst then syslog
set firewall family inet filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST}-dst then discard
set firewall family inet filter ${FIREWALL_FILTER} term Default-action then accept
EOF
)

    firewall_mx=$(cat <<EOF
set system syslog file ${LOG_FILENAME}.log firewall any
set policy-options prefix-list ${ALLOW_PREFIX_LIST}
set policy-options prefix-list ${BLOCK_PREFIX_LIST}
set firewall filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST} from prefix-list ${ALLOW_PREFIX_LIST}
set firewall filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST} then syslog
set firewall filter ${FIREWALL_FILTER} term ${ALLOW_PREFIX_LIST} then accept
set firewall filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST} from prefix-list ${BLOCK_PREFIX_LIST}
set firewall filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST} then syslog
set firewall filter ${FIREWALL_FILTER} term ${BLOCK_PREFIX_LIST} then discard
set firewall filter ${FIREWALL_FILTER} term Default-action then accept
EOF
)
    firewall_srx="${firewall_mx}"

    # SRX only!
    zone=$(cat <<EOZ
set system syslog file ${LOG_FILENAME}.log any any
set system syslog file ${LOG_FILENAME}.log match RT_FLOW_SESSION
EOZ
)

    # common footer, must exit to top-level, due to "edit" in the header
    footer=$(cat <<EOF
exit
commit
exit
EOF
)

    echoMsg DEBUG "creating commands for ${FIREWALL_MODE}_${J_MODEL_BASE}"
    case "${FIREWALL_MODE}_${J_MODEL_BASE}" in

        filter_ex)

            all_cmds="${header}\n${firewall_ex}\n${footer}"
        ;;
        filter_mx)

            all_cmds="${header}\n${firewall_mx}\n${footer}"
        ;;
        filter_srx)

            all_cmds="${header}\n${firewall_srx}\n${footer}"
        ;;
        zone) # SRX only!

            all_cmds="${header}\n${zone}\n${footer}"
        ;;
        *)

            echoMsg ERROR "Configuration error; FIREWALL_MODE (${FIREWALL_MODE}) is unrecognized"
            return 1
    esac

    # issue the built-up commands
    callCLI "${all_cmds}"
    if [ $? -eq 0 ]; then

        output=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} '/^commit complete$/p;')
        if [ "${output}" = "" ]; then

            echoMsg ERROR "${output}"
            echoMsg ALERT "Issuing rollback"
            callCLI "configure\nrollback\nexit"
            rval=1
        else

            echoMsg DEBUG "${TS_GROUP_NAME} group created"
        fi
    else

        echoMsg ERROR "${TS_GROUP_NAME} group creation failed"
        rval=1
    fi

    return $rval
}

# deleteGroup()
# returns
# 0 for success
# 1 for failure
deleteGroup() {

    local destroy
    local rval=0
    local output

    destroy=$(cat <<EOD
configure
delete groups ${TS_GROUP_NAME}
commit
exit
EOD
)

    callCLI "${destroy}"
    if [ $? -eq 0 ]; then

        output=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} "/apply-groups ${TS_GROUP_NAME}/p;")
        if [ "${output}" != "" ]; then

            echoMsg ERROR "${output}"
            echoMsg ALERT "Issuing rollback"
            callCLI "configure\nrollback\nexit"
            rval=1
        else

            echoMsg DEBUG "${TS_GROUP_NAME} group deleted"
        fi
    else

        echoMsg ERROR "${TS_GROUP_NAME} group deletion failed"
        rval=1
    fi

    return $rval
}

# loadList()
# Use "load update <file> relative" to have the device incorporate the latest
# ThreatSTOP address lists.
#
# args
# - full path to file in which the list exists
#
# returns :
# 0 for success
# 1 for failure
loadList() {

    local target_file=$1
    local prefix_list_name=$2
    local rval=0
    local cmd

    echoMsg DEBUG "loadList():"

    # Note, srx requires commit come from top level, so the
    # correct number of "exit" lines must exist
    cmd=$(cat <<EOC
configure
edit groups ${TS_GROUP_NAME}
edit policy-options prefix-list ${prefix_list_name}
load update ${target_file} relative
exit
exit
commit
exit
EOC
)

    callCLI "${cmd}"
    if [ $? -eq 0 ]; then

        output=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} '/^commit complete$/p;')
        if [ "${output}" = "" ]; then

            echoMsg ERROR "${output}"
            echoMsg ALERT "Issuing rollback"
            callCLI "configure\nrollback\nexit"
            rval=1
        else

            echoMsg DEBUG "\"load update\" succeeded"
        fi
    else

        rval=1
        echoMsg ERROR "Problem calling the CLI for \"load update\""
    fi

    return $rval
}

# loads globals :
# J_MODEL
# J_MODEL_BASE
# J_VERSION
loadJunosGlobals() {

    local ha_prefix # 'primary' or 'secondary' (for SRX), or
                    # 'master'  or 'backup'    (for EX/MX)
    local ha_role   # for passing to "show version ..."
                    # 'node'   (for SRX), or
                    # 'member' (for EX/MX)
    local ha_id     # 0, 1, 2, ...
    local old_ifs
    local j_version

    # HA_SAFE_TO_PROCEED, stored config variable, will set to either
    #       YES (should update) or NO (should not update)
    # ha_role, not stored, used to issue "show version"
    #       explicitly for this device
    callCLI "exit"
    if [ $? -ne 0 ]; then

        echoMsg ERROR "Problems calling the CLI : ${CLI_OUTPUT}"
    fi

    echoMsg DEBUG "CLI OUTPUT: $CLI_OUTPUT"
    ha_prefix=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} 's/^{(master|backup|primary|secondary):node([0-9]+).*}$/\1/p;')
    ha_id=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} 's/^{(master|backup|primary|secondary):node([0-9]+).*}$/\2/p;')

    case "${ha_prefix}" in

        primary) # SRX
            HA_SAFE_TO_PROCEED=YES
            ha_role="node ${ha_id}"
        ;;
        secondary) # SRX
            HA_SAFE_TO_PROCEED=NO
            ha_role="node ${ha_id}"
        ;;
        master) # EX/MX
            HA_SAFE_TO_PROCEED=YES
            ha_role="member ${ha_id}"
        ;;
        backup) # EX/MX
            HA_SAFE_TO_PROCEED=NO
            ha_role="member ${ha_id}"
        ;;
        *) # not in HA mode
            HA_SAFE_TO_PROCEED=YES
            ha_role=""
    esac

    echoMsg DEBUG "HA_SAFE_TO_PROCEED is '${HA_SAFE_TO_PROCEED}'"
    echoMsg DEBUG "ha_role is '${ha_role}'"
    if NotEmpty $ha_role;then
        echo "$ha_role" > "./.HA_ROLE"
    fi


    # load global version variables
    callCLI "show version ${ha_role}"
    if [ $? -eq 0 ]; then

        old_ifs="${IFS}"
        IFS=
        J_MODEL=$(echo "${CLI_OUTPUT}" |  sed -n ${SED_EXT_REGEXP} 's/^Model: (.+)$/\1/p;' |  awk '{ print tolower($0) }')
        # tr & head not found on some SRX's
        echoMsg DEBUG "Set JUNOS model to ${J_MODEL}"
        J_MODEL_BASE=$(echo "${J_MODEL}" | sed -n ${SED_EXT_REGEXP} 's/^v?([a-z]+).*$/\1/p;')
        echoMsg DEBUG "Set JUNOS model base to ${J_MODEL_BASE}"

        echo "${J_MODEL}" | grep -q ${SED_EXT_REGEXP} '^srx'
        if [ $? -eq 0 ]; then

            j_version=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} 's/^JUNOS Software Release \[(.+)\]$/\1/p;')
        else

            j_version=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} 's/^JUNOS Base OS boot \[(.+)\]$/\1/p;')
        fi
        IFS="${old_ifs}"

        if [ "${j_version}" != "" ]; then

            J_VERSION="${j_version}"
            echoMsg DEBUG "Set JUNOS version to ${J_VERSION}"
        fi
    else

        echoMsg ERROR "Unable to probe device for version information"
    fi
}

# checkSyslogConfig()
# probe device for syslog parameters
#
# if successful, the following globals can be assumed
# to have useful values
# - SYSLOG_OUT_FILE
# - SYSLOG_OUT_FTP_URL
#
# returns
# 0 for success
# 1 for failure
checkSyslogConfig() {

    local rval=0
    local file
    local ftp_url

    SYSLOG_OUT_FILE=""
    SYSLOG_OUT_FTP_URL=""

    callCLI "show configuration groups ${TS_GROUP_NAME} system syslog"

    if [ $? -eq 0 ]; then

        file=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} 's/^file ([^ ]+) {$/\1/p;')
        echoMsg DEBUG "Probed syslog file '${file}'"
        if [ "x${file}" != "x" ]; then

             SYSLOG_OUT_FILE=${file}
        fi

        ftp_url=$(echo "${CLI_OUTPUT}" | sed -n ${SED_EXT_REGEXP} 's/^ *"((pasv)?ftp:\/\/[^"]+)".*$/\1/p;')
        echoMsg DEBUG "Probed syslog file ftp url '${ftp_url}'"
        if [ "x${ftp_url}" != "x" ]; then

             SYSLOG_OUT_FTP_URL=${ftp_url}
        fi
    else

        echoMsg ERROR "Unable to call CLI for existing syslog configuration"
        rval=1
    fi

    return $rval
}

backup_existing_config() {
    if [ ! -f './pre-threatstop' ];then
        echoMsg INFO "Creating a backup configuration 'pre-threatstop'"
        if callCLI "configure\nsave pre-threatstop\nexit\nexit";then
            return 0;
        else
            return 1;
        fi
    fi
}

# set crontab
# 1. clean any of ours out
# 2. add ours back in
# 3. add a call to this in /opt/etc/rc.dtrace
#    (covers reboot)
# the /opt/etc/rc.dtrace is a mechanism to keep readding our updates to crontab upon reboots
# this issue does not seem to happen on EX line where you may see readonly fs issues. cron survives reboots fine.
# return
# 0 for success
# 1 for failure
installCron() {

    local all_cron
    local our_cron
    local cronPIDs
    local now_minute
    local rc_dtrace_file=/opt/etc/rc.dtrace
    local install_cron_script="${INSTALL_DIR}/install_cron.sh"
    # on EX possible write include to /var/etc/rc.conf.inc
    local rc_dtrace_content
    local rval=0

    echoMsg START "Install Cron"

    now_minute=$(date +%M)
    # if minute is 0 and we schedule tsadmin update 1 min prior lets ensure its never 00
    if [ $now_minute -eq 0 ];then
        now_minute=2
    fi
    every_5_mins='0,5,10,15,20,25,30,35,40,45,50,55'
    cronPIDs=$(ps aux | grep cron | grep -v grep | awk '{print $2}'| wc -l)
    ctp_update_script_or_nothing=''
    if [ "x${AUTO_TYPE}" = 'xauto' ];then
        echoMsg DEBUG "detected auto config mode"
        ctp_update_script_or_nothing="${every_5_mins} * * * * cd ${INSTALL_DIR}; /bin/sh ./ts-update-settings update 2>/dev/null 2>&1"
    fi


    if [ ${cronPIDs} -eq 0 ]; then

        echoMsg ERROR "Cron is not running; unable to install periodic update of ThreatSTOP address lists"
        return 1
    fi


    our_cron=$(cat <<EOC
# ThreatSTOP cron
${now_minute} */2 * * * /bin/sh ${INSTALL_DIR}/ts-update 2>/dev/null 2>&1
${ctp_update_script_or_nothing}
EOC
)
    echoMsg INFO "Adding update of policy address lists to cron"
    # get current cron, sans ours
    all_cron=$(crontab -l 2>/dev/null | grep -v ${SED_EXT_REGEXP} '(ThreatSTOP|ts-update|tsadmin)')

    # nuke all
    echo yes | crontab -r >/dev/null 2>&1
    if [ $? -ne 0 ]; then

        echoMsg DEBUG "Couldn't remove cron; it likely doesn't exist yet"
    fi

    # reinstall others, plus ours
    echo -e "${all_cron}\n${our_cron}" | crontab -
    if [ $? -ne 0 ]; then

        echoMsg ERROR "Couldn't install cron"
        rval=1
    fi

    echoMsg DEBUG "Installed cron\n${all_cron}\n${our_cron}"

    # the EX line doesn't seem to require this crontab rc hack
    if [ "[${J_MODEL_BASE}]" = "[ex]" ];then
        echoMsg DEBUG "Skipping rc.local crontab routine, not needed for EX line"
    else
        echoMsg INFO "Ensuring block/allow updates will happen after a reboot"
        mkdir -p $(dirname ${rc_dtrace_file})
        if [ $? -ne 0 ]; then

            echoMsg ERROR "Couldn't create directory for ${rc_dtrace_file}"
            rval=1
        fi

        install_crontab_script=". ${INSTALL_DIR}/ts-common.sh; . ${INSTALL_DIR}/ts-junos.sh; LOCKFILE=${INSTALL_DIR}/"
        install_crontab_script=${install_crontab_script}'$(basename $0).lock; setLock || exit 1; loadConfig; installCron; unLock'
        if [ -s ${rc_dtrace_file} ]; then

            grep -q 'installCron' ${rc_dtrace_file}
            if [ $? -ne 0 ]; then

                echoMsg DEBUG "Adding to ${rc_dtrace_file} :\n${install_crontab_script}"
                echo "export INSTALL_DIR=${INSTALL_DIR}; (${install_crontab_script})" >> ${rc_dtrace_file}
            fi
        else

            echoMsg DEBUG "Creating ${rc_dtrace_file} with :\n(${install_crontab_script})"
            echo "export INSTALL_DIR=${INSTALL_DIR}; (${install_crontab_script})" > ${rc_dtrace_file}
        fi
        # presense of .HA_ROLE means this device is part of a cluster
        # we should create a helper script to get crontab installed on secondary nodes
        if [ -f "${INSTALL_DIR}/.HA_ROLE" ] && [ ! -f ${install_cron_script} ];then
            # we are an HA cluster, write out a helper script to install crontab
            echo "#!/bin/sh " > ${install_cron_script};
            echo "export INSTALL_DIR=${INSTALL_DIR}; " >> ${install_cron_script};
            echo "cd ${INSTALL_DIR};" >> ${install_cron_script};
            echo "${install_crontab_script}" >> ${install_cron_script};
        fi
    fi

    return $rval
}

# remove crontab
# 1. clean any of ours out
# 2. remove anything we put in /opt/etc/rc.dtrace
#
# return
# 0 for success
# 1 for failure
removeCron() {

    local all_cron
    local rc_dtrace_file=/opt/etc/rc.dtrace

    echoMsg INFO "Removing ThreatSTOP from cron"
    # get current cron, sans ours
    all_cron=$(crontab -l 2>/dev/null | grep -v ${SED_EXT_REGEXP} '(ThreatSTOP|ts-update|tsadmin)')

    # nuke all
    echo yes | crontab -r >/dev/null 2>&1
    if [ $? -ne 0 ]; then

        echoMsg DEBUG "Couldn't remove cron; it likely doesn't exist yet"
    fi

    # reinstall remaining
    echo -e "${all_cron}" | crontab -
    if [ $? -ne 0 ]; then

        echoMsg DEBUG "Couldn't install cron"
        rval=1
    fi

    echoMsg INFO "Removing cron installation from boot script"
    if [ -s ${rc_dtrace_file} ]; then

        grep -q 'ts-common' ${rc_dtrace_file}
        if [ $? -eq 0 ]; then

            # remove our installed line
            sed -i.bak '/ts-common/d;' ${rc_dtrace_file}
        fi
    fi

    return $rval
}
