#
# Copyright(c) 2006-2019 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

# file : ts-prompt.sh
# Routines to prompt the user for input.
#

# _promptGeneric() :
# Interactively collect from the user the data-driven input
#
# args :
# - prompt string
# - error string
# - validation regex (optional)
# - invalidation regex, representing invalid characters (usually
#   written as a negated character class...[^0-9], for example,
#   ...also optional)
# - default value
#
# globals referenced :
# PROMPT_OUT - sets with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
_promptGeneric() {

    local prompt_str="$1"
    local error_str="$2"
    local valid_regex="$3"
    local invalid_regex="$4"
    local default="$5"
    local rval=0
    local input

    while true; do

        read -p "${prompt_str}" input

        echoMsg DEBUG "input from user : '${input}'"

        if [ "${input}" = "" ]; then

            input="${default}"
        fi

        if [ "${valid_regex}" != "" ]; then

            echo "${input}" | grep -q -E "${valid_regex}"
            if [ $? -ne 0 ]; then

                echoMsg WARN "${error_str}"
                continue
            fi
        fi

        if [ "${invalid_regex}" != "" ]; then

            echo "${input}" | grep -q -E "${invalid_regex}"
            if [ $? -eq 0 ]; then

                echoMsg WARN "${error_str}"
                continue
            fi
        fi

        break
    done

    PROMPT_OUT="$input"

    return $rval
}

# promptPolicy() :
# Interactively collect from the user the name of block or
# allow policy zone, based upon the passed type.
#
# args :
# - type (block, allow, ...)
# - default (current value for this policy zone, if any)
#
# globals referenced :
# PROMPT_OUT - sets with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
promptPolicy() {

    local type=$1
    local default=$2
    local rval=0
    local zone
    local prompt="$type policy domain"

    if [ "${default}" != "" ]; then

        prompt="${prompt} (hit return for ${default})"
    fi
    prompt="${prompt} : "

    _promptGeneric \
        "${prompt}" \
        "Invalid domain name; limit input to alpha-numeric, dot(.), and hyphen(-)" \
        "" \
        '[^-.a-zA-Z0-9]' \
        "${default}"

    return $rval
}

# promptValidPolicy()
# This will continue to ask for a policy until
# 1. it is of valid characters, or the old one is selected
# 2. AND it is query-able for one or more PTR records
# If the user wants out of this loop, they'll have to hit ctrl-C
#
# uses ts-dns.sh::queryPolicyZone
#
# args :
# - type, for display in the prompt
# - existing policy domain
#
# returns
# 0 - they either enter a valid, queryable policy, or they exit setup
promptValidPolicy() {

    local type=$1
    local policy=$2

    while true; do

        promptPolicy $type $policy
        queryPolicyZone "${PROMPT_OUT}"
        if [ $? -ne 0 ]; then

            echoMsg ALERT "Enter a valid policy zone, or hit CTRL-C to exit setup"
            continue
        fi

        echoMsg DEBUG "Policy success : '${PROMPT_OUT}'"
        break
    done
}

# promptValidPort() :
# Interactively collect from the user the maximum number
# of CIDRs that we'll present to their device
#
# args :
# - default (current max)
#
# globals referenced :
# PROMPT_OUT - sets this with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
promptValidPort() {

    local default=$1
    local rval=0

    _promptGeneric \
        "Enter DNS Port (hit return for ${default}) : " \
        "Invalid Port; Valid ports (53, 5353)" \
        '^53$|^5353$' \
        "" \
        "${default}"

    return $rval
}

# promptMaxPolicySize() :
# Interactively collect from the user the maximum number
# of CIDRs that we'll present to their device
#
# args :
# - default (current max)
#
# globals referenced :
# PROMPT_OUT - sets this with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
promptMaxPolicySize() {

    local default=$1
    local rval=0

    _promptGeneric \
        "Enter maximum policy size (hit return for ${default}) : " \
        "Invalid size; limit input to integer values" \
        "" \
        '[^0-9]' \
        "${default}"

    return $rval
}

# promptLogUploadIP() :
# Interactively collect from the user the IP address that
# logs should be associated with, as configured in the
# customer's portal account for this device
#
# args :
# - default (current IP)
#
# globals referenced :
# PROMPT_OUT - sets this with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
promptLogUploadIP() {

    local default=$1
    local rval=0
    local prompt="Enter the IP address with which logs should be associated"

    if [ "${default}" != "" ]; then

        prompt="${prompt} (hit return for ${default})"
    fi
    prompt="${prompt} : "

    _promptGeneric \
        "${prompt}" \
        "Invalid IP; limit input to dotted-quad (192.168.1.1) notation" \
        '^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$' \
        "" \
        "${default}"

    return $rval
}
# promptLogUploadTDID() :
# Interactively collect TDID from the user
# logs should be associated with, as configured in the
# customer's portal account for this device
#
# args :
# - default (current TDID)
#
# globals referenced :
# PROMPT_OUT - sets this with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
promptLogUploadTDID() {

    local default=$1
    local rval=0
    local prompt="Enter the ThreatSTOP Device ID (TDID)"

    if [ "${default}" != "" ]; then

        prompt="${prompt} (hit return for ${default})"
    fi
    prompt="${prompt} : "

    _promptGeneric \
        "${prompt}" \
        "Invalid TDID; Your TDID begins with 'tdid_' and is available on the portal under the device info section." \
        '^tdid\_[a-zA-Z0-9]+$' \
        "" \
        "${default}"

    return $rval
}

# promptFirewallMode() :
# The customer needs to tell us if their device is
# configured to use "filter" (prefix lists) or "zone"
# (address books)
#
# args :
# - default (filter or zone)
#
# globals referenced :
# PROMPT_OUT - sets this with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
promptFirewallMode() {

    local default=$1
    local rval=0

    _promptGeneric \
        "Enter firewall mode, \"filter\" for prefix lists, or \"zone\" for address books (hit return for \"${default}\") : " \
        "Invalid firewall mode; limit input to \"filter\" or \"zone\"" \
        '^(filter|zone)$' \
        "" \
        "${default}"

    return $rval
}

# promptYesNo() :
# Prompt with the passed string, and return "yes" or "no"
#
# args :
# - default (yes or no)
#
# globals referenced :
# PROMPT_OUT - sets this with the result of the collected information
#
# returns :
# - loads PROMPT_OUT with collection results
# - returns 0
#
promptYesNo() {

    local prompt="$1"
    local default=$2
    local rval=0

    _promptGeneric \
        "${prompt}" \
        "type \"yes\" or \"no\"" \
        '^(yes|Yes|YEs|YES|yES|yeS|no|No|NO|nO)$' \
        "" \
        "${default}"

    PROMPT_OUT=$(echo ${PROMPT_OUT} | awk '{print tolower($0)}');
    return $rval
}
